﻿using System;
using System.Collections.Generic;
using GenericMethods;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace GenericMethodsTests
{
    [TestClass]
    public class MathGenerichTests
    {
        [TestMethod]
        public void MakeListTest()
        {
            List<String> list = MathGeneric.MakeList("Anders");
            Assert.AreEqual(1, list.Count);
            Assert.IsTrue(list.Contains("Anders"));
            //List<String> list2 = MathGeneric.MakeList(3);
        }

        [TestMethod]
        public void MinMaxComparableTest()
        {
            
            Student st1 = new Student { Id = 23, Name = "Anders" };
            Student st2 = new Student { Id=56, Name = "John" };
            Assert.AreEqual(st1, MathGeneric.Min(st1, st2));
            Assert.AreEqual(st1, MathGeneric.Min(st2, st1));

            Assert.AreEqual(st2, MathGeneric.Max(st1, st2));
            Assert.AreEqual(st2, MathGeneric.Max(st2, st1));

        }

        [TestMethod]
        public void MinMaxComparerTest()
        {
            Assert.AreEqual(3, MathGeneric.Min(3, 6));
            Student st1 = new Student { Name = "Anders" };
            Student st2 = new Student { Name = "John" };
            Assert.AreEqual(st1, MathGeneric.Min(st1, st2, Student.NameComparer));
            Assert.AreEqual(st1, MathGeneric.Min(st2, st1, Student.NameComparer));

            Assert.AreEqual(st2, MathGeneric.Max(st1, st2, Student.NameComparer));
            Assert.AreEqual(st2, MathGeneric.Max(st2, st1, Student.NameComparer));
       
        }

        [TestMethod]
        public void MinMaxTestDelegate()
        {
            Comparison<int> comp = (a, b) => a - b;

            Assert.AreEqual(3, MathGeneric.Min(5, 3, comp));
            Assert.AreEqual(4, MathGeneric.Min(4, 6, comp));

            Assert.AreEqual(12, MathGeneric.Max(12, 5, comp));
            Assert.AreEqual(44, MathGeneric.Max(22, 44, comp));
        }

        [TestMethod]
        public void FrequencyMapTest()
        {
            String[] words = { "method", "can", "give", "another", "method", "a", "name" };
            IDictionary<String, int> dictionary = MathGeneric.Frequencies(words);
            Assert.AreEqual(2, dictionary["method"]);
            Assert.AreEqual(1, dictionary["can"]);
        }

        [TestMethod]
        public void IntersectionTest()
        {
            ICollection<int> coll1 = new[] { 4, 2, 6, 3, 22 };
            ICollection<int> coll2 = new[] { 5, 3, 23, 8, 2, 2, 4 };
            ICollection<int> intersect = MathGeneric.InterSection(coll1, coll2);
            Assert.AreEqual(3, intersect.Count);
            Assert.IsTrue(intersect.Contains(2));
            Assert.IsTrue(intersect.Contains(4));
            Assert.IsTrue(intersect.Contains(3));
        }
    }

    class Student : IComparable<Student>
    {
        public static readonly IComparer<Student> NameComparer = new StudentComparer();


        public String Name { get; set; }
        public int Id { get; set; }
        public int CompareTo(Student other)
        {
            return this.Id - other.Id;
        }

        public override string ToString()
        {
            return string.Format("Id: {0}, Name: {1}", Id, Name);
        }
    }

    class StudentComparer : IComparer<Student>
    {
        public int Compare(Student x, Student y)
        {
            return x.Name.CompareTo(y.Name);
        }
    }

}
